-- Author      : Nogarder
-- Create Date : 2015/3/2 1:34:45
local pairs=pairs
local CopyTable=CopyTable
--
local KeyWordColor="|cffff0000"
local ValueColor="|cff00ff00"
local OtherColor="|cffffffff"
--localization
local Localization=RS.Localization
local Tooltips=Localization.Tooltips
--macro for creating unused name
local FindName=function(list,name)
	for _,button in pairs(list.Buttons)do
		if(button:IsShown() and button.Text==name)then
			return true
		end
	end
end
local CheckName=function(list,name)
	local newname=name
	if(FindName(list,newname))then
		local i=1
		while(FindName(list,newname))do
			newname=name.." ("..i..")"
			i=i+1
		end
	end
	return newname
end
--
RS.Template.UpdateListEditor=function(list,data)
	RS.Template.UpdateSelectList(list,data)
	RS.ClosePages(list.Parent,true)
end

---
RS.Template.CreateListEditor=function(frame,title,width,height,parent,buttonheight,nomenu,menuparent)
	local selectlist=RS.Template.CreateSelectList(parent,width,height,title,tooltips,frame,buttonheight,true)
	--
	selectlist.Parent=parent
	--
	local OnCreateFunc=function()
		local inputbox=RS.Template.SetInputBox(menuparent or selectlist,Localization["InputNewName"],"")
		inputbox.CallBackFunc=function(newname)
			local newname=CheckName(selectlist.Frame,newname)
			RS.RunFunc(selectlist.OnCreateFunc,newname)
			RS.RunFunc(selectlist.RefreshFunc,selectlist)
			--set focus
			RS.Template.SetSelectListFocus(selectlist,newname)
			--
		end
		inputbox:ClearAllPoints()
		inputbox:SetPoint("BOTTOMRIGHT",selectlist,"TOPRIGHT",0,13)
		inputbox:SetFocus()
	end
	local OnDeleteFunc=function(value)
		if(selectlist.NoDefault==nil and value=="Default")then
			return
		end
		RS.RunFunc(selectlist.OnDeleteFunc,value)
		RS.RunFunc(selectlist.RefreshFunc,selectlist)
	end
	local OnCopyFunc=function(value)
		local newname=CheckName(selectlist.Frame,value..Localization["sCopy"])
		RS.RunFunc(selectlist.OnCopyFunc,value,newname)
		RS.RunFunc(selectlist.RefreshFunc,selectlist)
	end
	--
	selectlist.Frame.OnCopyFunc=OnCopyFunc
	selectlist.Frame.OnDeleteFunc=OnDeleteFunc
	selectlist.Frame.OnAddFunc=OnCreateFunc
	--dropdownmenu
	if(nomenu)then
		selectlist.ButtonClickFunc=function(self,button)
			RS.RunFunc(selectlist.OnClickFunc,selectlist.Value,button,self)
		end
	else
		--
		local buttondata={
			[1]={
				Text=Localization["Copy"],
				Value="Copy",
				Tooltips=Tooltips["Copy"]
			},
			[2]={
				Text=Localization["Delete"],
				Value="Delete"
			},
			[3]={
				Text=Localization["Rename"],
				Value="Rename"
			},
			[4]={
				Text=Localization["Create"],
				Value="Create",
				Tooltips=Tooltips["Create"],
			}
		}
		local RS_Template_ListEditor_DropDownMenu_OnClickFunc=function(self)
			local value=self.Value
			if(value=="Copy")then
				OnCopyFunc(selectlist.Value)
			elseif(value=="Delete")then
				OnDeleteFunc(selectlist.Value)
			elseif(value=="Rename")then
				local inputbox=RS.Template.SetInputBox(menuparent or selectlist,Localization["InputNewName"],selectlist.Value)
				inputbox.CallBackFunc=function(newname)
					local newname=CheckName(selectlist.Frame,newname)
					RS.RunFunc(selectlist.OnRenameFunc,selectlist.Value,newname)
					RS.RunFunc(selectlist.RefreshFunc,selectlist)
				end
				inputbox:ClearAllPoints()
				inputbox:SetPoint("BOTTOM",selectlist,"TOP",0,13)
				inputbox:SetFocus()
			elseif(value=="Create")then
				OnCreateFunc()
			end
		end
		--
		selectlist.ButtonClickFunc=function(self,button)
			RS.Template.HideInputBox()
			if(button=="RightButton")then
				if(selectlist.NoRename==true)then
					buttondata[3].Tooltips=""
					buttondata[3].Disabled=true
				else
					buttondata[3].Tooltips=Tooltips["Rename"]
					buttondata[3].Disabled=nil
				end
				if(selectlist.NoDelete==true)then
					buttondata[2].Tooltips=""
					buttondata[2].Disabled=true
				else
					buttondata[2].Tooltips=Tooltips["Delete"]
					buttondata[2].Disabled=nil
				end
				if(selectlist.NoCopy==true)then
					buttondata[1].Tooltips=""
					buttondata[1].Disabled=true
				else
					buttondata[1].Tooltips=Tooltips["Copy"]
					buttondata[1].Disabled=nil
				end
				local dropdownmenu=RS.Template.SetDropDownMenu(self:GetParent().Scroll,nil,buttondata,RS_Template_ListEditor_DropDownMenu_OnClickFunc,menuparent or selectlist)
				dropdownmenu:ClearAllPoints()
				dropdownmenu:SetPoint("LEFT",self:GetParent().Scroll,"RIGHT",3,0)
				dropdownmenu:SetPoint("TOP",self,"TOP",3,0)
			else
				RS.Template.HideDropDownMenu()
			end
			RS.RunFunc(selectlist.OnClickFunc,selectlist.Value,button,self)
		end
		local frameonclick=function(_,button)
			if(button=="RightButton")then
				buttondata[1].Disabled=true
				buttondata[2].Disabled=true
				buttondata[2].Tooltips=Tooltips["Delete"]
				buttondata[3].Disabled=true
				buttondata[3].Tooltips=Tooltips["Rename"]
				local dropdownmenu=RS.Template.SetDropDownMenu(selectlist,nil,buttondata,RS_Template_ListEditor_DropDownMenu_OnClickFunc,menuparent or selectlist)
				dropdownmenu:ClearAllPoints()
				dropdownmenu:SetPoint("LEFT",selectlist,"RIGHT",3,0)
				dropdownmenu:SetPoint("TOP",selectlist,"TOP",3,0)
			else
				RS.Template.HideDropDownMenu()
			end
		end
		selectlist:HookScript("OnMouseUp",frameonclick)
		--
		selectlist.Frame:HookScript("OnMouseUp",frameonclick)
	end
	--
	parent:HookScript("OnShow",function()
			RS.RunFunc(selectlist.RefreshFunc,selectlist)
		end)

	return selectlist
end

